@@ -1,20 +1,81 @@
 r"""
 Estimate entropy after a fit.
 
-The :func:`entropy` method computes the entropy directly from a set of
-MCMC samples, normalized by a scale factor computed from the kernel density
-estimate at a subset of the points.\ [#Kramer]_
+The :func:`gmm_entropy` function computes the entropy from a Gaussian mixture
+model. This provides a reasonable estimate even for non-Gaussian distributions.
+This is the recommended method for estimating the entropy of a sample.
 
 The :func:`cov_entropy` method computes the entropy associated with the
 covariance matrix.  This covariance matrix can be estimated during the
 fitting procedure (BFGS updates an estimate of the Hessian matrix for example),
 or computed by estimating derivatives when the fit is complete.
 
-The :class:`MVNEntropy` estimates the covariance from an MCMC sample and
+The :class:`MVNEntropy` class estimates the covariance from an MCMC sample and
 uses this covariance to estimate the entropy.  This gives a better
 estimate of the entropy than the equivalent direct calculation, which requires
 many more samples for a good kernel density estimate.  The *reject_normal*
 attribute is *True* if the MCMC sample is significantly different from normal.
+Unfortunately, this almost always the case for any reasonable sample size that
+isn't strictly gaussian.
+
+The :func:`entropy` function computes the entropy directly from a set
+of MCMC samples, normalized by a scale factor computed from the kernel density
+estimate at a subset of the points.\ [#Kramer]_
+
+There are many other entropy calculations implemented within this file, as
+well as a number of sampling distributions for which the true entropy is known.
+Furthermore, entropy was computed against dream output and checked for
+consistency. None of the methods is truly excellent in terms of minimum
+sample size, maximum dimensions and speed, but many of them are pretty
+good.
+
+The following is an informal summary of the results from different algorithms
+applied to DREAM output::
+
+        from .entropy import Timer as T
+
+        # Try MVN ... only good for normal distributions, but very fast
+        with T(): M = entropy.MVNEntropy(drawn.points)
+        print("Entropy from MVN: %s"%str(M))
+
+        # Try wnn ... no good.
+        with T(): S_wnn, Serr_wnn = entropy.wnn_entropy(drawn.points, n_est=20000)
+        print("Entropy from wnn: %s"%str(S_wnn))
+
+        # Try wnn with bootstrap ... still no good.
+        with T(): S_wnn, Serr_wnn = entropy.wnn_bootstrap(drawn.points)
+        print("Entropy from wnn bootstrap: %s"%str(S_wnn))
+
+        # Try wnn entropy with thinning ... still no good.
+        #drawn = self.draw(portion=portion, vars=vars,
+        #                  selection=selection, thin=10)
+        with T(): S_wnn, Serr_wnn = entropy.wnn_entropy(points)
+        print("Entropy from wnn: %s"%str(S_wnn))
+
+        # Try wnn with gmm ... still no good
+        with T(): S_wnn, Serr_wnn = entropy.wnn_entropy(drawn.points, n_est=20000, gmm=20)
+        print("Entropy from wnn with gmm: %s"%str(S_wnn))
+
+        # Try pure gmm ... pretty good
+        with T(): S_gmm, Serr_gmm = entropy.gmm_entropy(drawn.points, n_est=10000)
+        print("Entropy from gmm: %s"%str(S_gmm))
+
+        # Try kde from statsmodels ... pretty good
+        with T(): S_kde_stats = entropy.kde_entropy_statsmodels(drawn.points, n_est=10000)
+        print("Entropy from kde statsmodels: %s"%str(S_kde_stats))
+
+        # Try kde from sklearn ... pretty good
+        with T(): S_kde = entropy.kde_entropy_sklearn(drawn.points, n_est=10000)
+        print("Entropy from kde sklearn: %s"%str(S_kde))
+
+        # Try kde from sklearn at points from gmm ... pretty good
+        with T(): S_kde_gmm = entropy.kde_entropy_sklearn_gmm(drawn.points, n_est=10000)
+        print("Entropy from kde+gmm: %s"%str(S_kde_gmm))
+
+        # Try Kramer ... pretty good, but doesn't support marginal entropy
+        with T(): S, Serr = entropy.entropy(drawn.points, drawn.logp, N_entropy=n_est)
+        print("Entropy from Kramer: %s"%str(S))
+
 
 .. [#Kramer]
     Kramer, A., Hasenauer, J., Allgower, F., Radde, N., 2010.
@@ -48,11 +109,11 @@ attribute is *True* if the MCMC sample is significantly different from normal.
 """
 from __future__ import division, print_function
 
-__all__ = ["entropy"]
+__all__ = ["entropy", "gmm_entropy", "cov_entropy", "wnn_entropy", "MVNEntropy"]
 
 import numpy as np
 from numpy import mean, std, exp, log, sqrt, log2, pi, e, nan
-from numpy.random import permutation
+from numpy.random import permutation, choice
 from scipy import stats
 from scipy.stats import norm, chi2
 from scipy.special import gammaln, digamma
@@ -166,17 +227,41 @@ def kde_entropy_sklearn_gmm(points, n_est=None, n_components=None):
     return H / LN2
 
 def gmm_entropy(points, n_est=None, n_components=None):
+    r"""
+    Use sklearn.mixture.BayesianGaussianMixture to estimate entropy.
+
+    *points* are the data points in the sample.
+
+    *n_est* are the number of points to use in the estimation; default is
+    10,000 points, or 0 for all the points.
+
+    *n_components* are the number of Gaussians in the mixture. Default is
+    $5 \sqrt{d}$ where $d$ is the number of dimensions.
+
+    Returns estimated entropy and uncertainty in the estimate.
+
+    This method uses BayesianGaussianMixture from scikit-learn to build a
+    model of the point distribution, then uses Monte Carlo sampling to
+    determine the entropy of that distribution. The entropy uncertainty is
+    computed from the variance in the MC sample scaled by the number of
+    samples. This does not incorporate any uncertainty in the sampling that
+    generated the point distribution or the uncertainty in the GMM used to
+    model that distribution.
+    """
     #from sklearn.mixture import GaussianMixture as GMM
     from sklearn.mixture import BayesianGaussianMixture as GMM
     n, d = points.shape
 
     # Default to the full set
     if n_est is None:
+        n_est = 10000
+    elif n_est == 0:
         n_est = n
 
     # reduce size of draw to n_est
     if n_est >= n:
         x = points
+        n_est = n
     else:
         x = points[permutation(n)[:n_est]]
         n = n_est
@@ -195,7 +280,7 @@ def gmm_entropy(points, n_est=None, n_components=None):
     weight_x = predictor.score_samples(eval_x)
     H = -np.mean(weight_x)
     #with np.errstate(divide='ignore'): H = H + np.sum(np.log(sigma))   # if standardized
-    dH = 0.
+    dH = np.std(weight_x, ddof=1) / sqrt(n)
     ## cross-check against own calcs
     #alt = GaussianMixture(predictor.weights_, mu=predictor.means_, sigma=predictor.covariances_)
     #print("alt", H, alt.entropy())
@@ -203,7 +288,7 @@ def gmm_entropy(points, n_est=None, n_components=None):
     return H / LN2, dH / LN2
 
 def wnn_bootstrap(points, k=None, weights=True, n_est=None, reps=10, parts=10):
-    raise NotImplementedError("deprecated; bootstrap doesn't help.")
+    #raise NotImplementedError("deprecated; bootstrap doesn't help.")
     n, d = points.shape
     if n_est is None:
         n_est = n//parts
@@ -234,18 +319,21 @@ def wnn_entropy(points, k=None, weights=True, n_est=None, gmm=None):
 
     Berrett, T. B., Samworth, R.J., Yuan, M., 2016. Efficient multivariate
     entropy estimation via k-nearest neighbour distances.
-    https://arxiv.org/abs/1606.00304
+    DOI:10.1214/18-AOS1688 https://arxiv.org/abs/1606.00304
     """
     from sklearn.neighbors import NearestNeighbors
     n, d = points.shape
 
     # Default to the full set
     if n_est is None:
+        n_est = 10000
+    elif n_est == 0:
         n_est = n
 
     # reduce size of draw to n_est
     if n_est >= n:
         x = points
+        n_est = n
     else:
         x = points[permutation(n)[:n_est]]
         n = n_est
@@ -349,7 +437,7 @@ def _wnn_weights(k, d, weighted=True):
     sum_one = [[1.]*k]
     A = np.array(sum_zero + sum_one)
     b = np.array([[0.]*(d//4)+[1.]]).T
-    return np.dot(np.linalg.pinv(A),b)
+    return np.dot(np.linalg.pinv(A), b)
 
 def scipy_stats_density(sample_points, evaluation_points):  # pragma: no cover
     """
@@ -559,13 +647,13 @@ def entropy(points, logp, N_entropy=10000, N_norm=2500):
         idx = pylab.argsort(entropy_points[:, 0])
         pylab.figure()
         pylab.subplot(221)
-        pylab.hist(points[:, 0], bins=50, normed=True, log=True)
+        pylab.hist(points[:, 0], bins=50, density=True, log=True)
         pylab.plot(entropy_points[idx, 0], rho[idx], label='density')
         pylab.plot(entropy_points[idx, 0], exp(eval_logp+log_scale)[idx], label='p')
         pylab.ylabel("p(x)")
         pylab.legend()
         pylab.subplot(222)
-        pylab.hist(points[:, 0], bins=50, normed=True, log=False)
+        pylab.hist(points[:, 0], bins=50, density=True, log=False)
         pylab.plot(entropy_points[idx, 0], rho[idx], label='density')
         pylab.plot(entropy_points[idx, 0], exp(eval_logp+log_scale)[idx], label='p')
         pylab.ylabel("p(x)")
@@ -589,9 +677,10 @@ class MVNEntropy(object):
 
     *alpha* is the cutoff for the normality test.
 
-    *max_points* is the maximum number of points to use when computing the
-    entropy.  Since the normality test is $O(n^2)$ in memory and time,
-    where $n$ is the number of points, *max_points* defaults to 1000.
+    *max_points* is the maximum number of points to use when checking
+    normality.  Since the normality test is $O(n^2)$ in memory and time,
+    where $n$ is the number of points, *max_points* defaults to 1000. The
+    entropy is computed from the full dataset.
 
     The returned object has the following attributes:
 
@@ -606,6 +695,10 @@ class MVNEntropy(object):
         to the distribution
 
     """
+    # TODO: use robust covariance estimator for mean and covariance
+    # FastMSD is available in sklearn.covariance.MinDetCov. There are methods
+    # such as (Zhahg, 2012), which may be faster if performance is an issue.
+    # [1] Zhang (2012) DOI: 10.5539/ijsp.v1n2p119
     def __init__(self, x, alpha=0.05, max_points=1000):
         # compute Mardia test coefficient
         n, p = x.shape   # num points, num dimensions
@@ -647,6 +740,23 @@ def cov_entropy(C):
     """
     return 0.5 * (len(C) * log2(2*pi*e) + log2(abs(np.linalg.det(C))))
 
+
+def mvn_entropy_bootstrap(points, samples=50):
+    """
+    Use bootstrap method to estimate entropy and its uncertainty
+    """
+    n, d = points.shape
+
+    results = []
+    for _ in range(samples):
+        # sample n points with replacement in 0 ... n-1.
+        x = points[choice(n, size=n)]
+        C = np.cov(x.T, bias=1) if d > 1 else np.array([[np.var(x.T, ddof=1)]])
+        #print(f"cov {samples}, {x.shape}, {C.shape}")
+        results.append(cov_entropy(C))
+
+    return np.mean(results), np.std(results)
+
 # ======================================================================
 # Testing code
 # ======================================================================
